# Generated by Django 5.2 on 2025-04-23 12:46

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='CarePlan',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('status', models.CharField(choices=[('draft', 'Draft'), ('active', 'Active'), ('completed', 'Completed'), ('entered-in-error', 'Entered in Error'), ('unknown', 'Unknown'), ('revoked', 'Revoked'), ('on-hold', 'On Hold')], max_length=255, null=True)),
                ('intent', models.CharField(choices=[('proposal', 'Proposal'), ('plan', 'Plan'), ('order', 'Order'), ('option', 'Option'), ('directive', 'Directive')], max_length=255, null=True)),
                ('title', models.CharField(max_length=255, null=True)),
                ('description', models.TextField(null=True)),
                ('created', models.DateTimeField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='CarePlanActivity',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='CarePlanAddressesCodeableReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='CarePlanAddressReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='CarePlanBasedOnReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='CarePlanCustodianReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='CarePlanDeviceRequestReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='CarePlanPlannedActivityReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='CarePlanReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



"""medication statement serializers."""

from drf_writable_nested import WritableNestedModelSerializer

from dfhir.base.serializers import (
    AnnotationSerializer,
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    IdentifierSerializer,
    PeriodSerializer,
    ReferenceSerializer,
    TimingSerializer,
)
from dfhir.encounters.serializers import EncounterReferenceSerializer
from dfhir.medications.serializers import MedicationCodeableReferenceSerializer
from dfhir.medicationstatements.models import (
    MedicationStatement,
    MedicationStatementAdherence,
    MedicationStatementInformationSourceReference,
    MedicationStatementPartOfReference,
    MedicationStatementReasonReference,
)
from dfhir.patients.serializers import PatientGroupReferenceSerializer


class MedicationStatementPartOfReferenceSerializer(BaseReferenceModelSerializer):
    """medication statement part of serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """meta options."""

        model = MedicationStatementPartOfReference
        exclude = ["created_at", "updated_at"]


class MedicationStatementInformationSourceReferenceSerializer(
    BaseReferenceModelSerializer
):
    """medication statement information source reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """meta options."""

        model = MedicationStatementInformationSourceReference
        exclude = ["created_at", "updated_at"]


class MedicationStatementReasonReferenceSerializer(BaseReferenceModelSerializer):
    """medication statement reason reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """meta options."""

        model = MedicationStatementReasonReference
        exclude = ["created_at", "updated_at"]


class MedicationStatementAdherenceSerializer(WritableNestedModelSerializer):
    """medication statement adherence serializer."""

    code = CodeableConceptSerializer(required=False)
    reason = CodeableConceptSerializer(required=False)

    class Meta:
        """meta options."""

        model = MedicationStatementAdherence
        exclude = ["created_at", "updated_at"]


class MedicationStatementSerializer(BaseWritableNestedModelSerializer):
    """medication statement reference."""

    identifier = IdentifierSerializer(many=True, required=False)
    part_of = MedicationStatementPartOfReferenceSerializer(many=True, required=False)
    category = CodeableConceptSerializer(many=True, required=False)
    medication = MedicationCodeableReferenceSerializer(required=False)
    subject = PatientGroupReferenceSerializer(required=False)
    encounter = EncounterReferenceSerializer(required=False)
    effective_period = PeriodSerializer(required=False)
    effective_timing = TimingSerializer(required=False)
    information_source = MedicationStatementInformationSourceReferenceSerializer(
        many=True, required=False
    )
    derived_from = ReferenceSerializer(many=True, required=False)
    reason = MedicationStatementReasonReferenceSerializer(many=True, required=False)
    note = AnnotationSerializer(many=True, required=False)
    adherence = MedicationStatementAdherenceSerializer(many=False, required=False)

    class Meta:
        """meta options."""

        model = MedicationStatement
        exclude = ["created_at", "updated_at"]



"""practitioner role urls."""

from django.urls import path

from dfhir.practitionerroles import views

app_name = "practitionerroles"

urlpatterns = [
    path("practitionerrole/", views.PractitionerRoleListView.as_view()),
    path("practitionerrole/<int:pk>/", views.PractitionerRoleDetailView.as_view()),
    path("practitionerrole/code/", views.PractitionerRoleCodeListView.as_view()),
]



"""medication knowledge URL Configuration."""

from django.urls import path

from dfhir.medicationknowledges import views

app_name = "medicationknowledges"
urlpatterns = [
    path("medicationknowledges/", views.MedicationKnowledgeListView.as_view()),
    path(
        "medicationknowledges/<int:pk>/", views.MedicationKnowledgeDetailView.as_view()
    ),
]



"""Procedure models."""

from django.db import models

from dfhir.base.models import (
    Age,
    Annotation,
    BaseReference,
    CodeableConcept,
    Identifier,
    OrganizationReference,
    Period,
    Range,
    Reference,
    TimeStampedModel,
    Timing,
)

from . import choices


class ProcedureReference(BaseReference):
    """Procedure Reference model."""

    procedure = models.ForeignKey(
        "Procedure",
        on_delete=models.DO_NOTHING,
        related_name="procedure_reference",
        null=True,
    )
    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        related_name="procedure_reference_identifier",
        null=True,
    )


class ProcedureBasedOnReference(BaseReference):
    """Procedure Based On Reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        related_name="procedure_based_on_reference_identifier",
        null=True,
    )
    care_plan = models.ForeignKey(
        "careplans.CarePlan",
        on_delete=models.DO_NOTHING,
        related_name="procedure_based_on_reference_care_plan",
        null=True,
    )
    service_request = models.ForeignKey(
        "servicerequests.ServiceRequest",
        on_delete=models.DO_NOTHING,
        related_name="procedure_based_on_reference_service_request",
        null=True,
    )
    medication_request = models.ForeignKey(
        "medicationrequests.MedicationRequest",
        on_delete=models.DO_NOTHING,
        related_name="procedure_based_on_reference_medication_request",
        null=True,
    )


class ProcedurePartOfReference(BaseReference):
    """Procedure PartOf Reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        related_name="procedure_partof_reference_identifier",
        null=True,
    )
    procedure = models.ForeignKey(
        "Procedure",
        on_delete=models.DO_NOTHING,
        related_name="procedure_partof_reference_procedure",
        null=True,
    )
    observation = models.ForeignKey(
        "observations.Observation",
        on_delete=models.DO_NOTHING,
        related_name="procedure_partof_reference_observation",
        null=True,
    )
    medication_administration = models.ForeignKey(
        "medicationadministrations.MedicationAdministration",
        on_delete=models.DO_NOTHING,
        related_name="procedure_partof_reference_medication_administration",
        null=True,
    )


class ProcedureSubjectReference(BaseReference):
    """Procedure Subject Reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        related_name="procedure_subject_reference_identifier",
        null=True,
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.DO_NOTHING,
        related_name="procedure_subject_reference_patient",
        null=True,
    )
    group = models.ForeignKey(
        "groups.Group",
        on_delete=models.DO_NOTHING,
        related_name="procedure_subject_reference_group",
        null=True,
    )
    device = models.ForeignKey(
        "devices.Device",
        on_delete=models.DO_NOTHING,
        related_name="procedure_subject_reference_device",
        null=True,
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.DO_NOTHING,
        related_name="procedure_subject_reference_organization",
        null=True,
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.DO_NOTHING,
        related_name="procedure_subject_reference_practitioner",
        null=True,
    )
    location = models.ForeignKey(
        "locations.Location",
        on_delete=models.DO_NOTHING,
        related_name="procedure_subject_reference_location",
        null=True,
    )


class ProcedureFocusReference(BaseReference):
    """Procedure focus reference."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        related_name="procedure_focus_reference_identifier",
        null=True,
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.DO_NOTHING,
        related_name="procedure_focus_reference_patient",
        null=True,
    )
    group = models.ForeignKey(
        "groups.Group",
        on_delete=models.DO_NOTHING,
        related_name="procedure_focus_reference_group",
        null=True,
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.DO_NOTHING,
        related_name="procedure_focus_reference_related_person",
        null=True,
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.DO_NOTHING,
        related_name="procedure_focus_reference_practitioner",
        null=True,
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.DO_NOTHING,
        related_name="procedure_focus_reference_organization",
        null=True,
    )
    care_team = models.ForeignKey(
        "careteams.CareTeam",
        on_delete=models.DO_NOTHING,
        related_name="procedure_focus_reference_care_team",
        null=True,
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.DO_NOTHING,
        related_name="procedure_focus_reference_practitioner_role",
        null=True,
    )
    specimen = models.ForeignKey(
        "specimens.Specimen",
        on_delete=models.DO_NOTHING,
        related_name="procedure_focus_reference_specimen",
        null=True,
    )


class ProcedureRecorderReference(BaseReference):
    """Procedure Recorder Reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        related_name="procedure_recorder_reference_identifier",
        null=True,
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.DO_NOTHING,
        related_name="procedure_recorder_reference_patient",
        null=True,
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.DO_NOTHING,
        related_name="procedure_recorder_reference_related_person",
        null=True,
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.DO_NOTHING,
        related_name="procedure_recorder_reference_practitioner",
        null=True,
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.DO_NOTHING,
        related_name="procedure_recorder_reference_practitioner_role",
        null=True,
    )


class ProcedurePerformerActorReference(BaseReference):
    """Procedure Performer Actor Reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        related_name="procedure_performer_actor_reference_identifier",
        null=True,
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.DO_NOTHING,
        related_name="procedure_performer_actor_reference_practitioner",
        null=True,
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.DO_NOTHING,
        related_name="procedure_performer_actor_reference_practitioner_role",
        null=True,
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.DO_NOTHING,
        related_name="procedure_performer_actor_reference_organization",
        null=True,
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.DO_NOTHING,
        related_name="procedure_performer_actor_reference_patient",
        null=True,
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.DO_NOTHING,
        related_name="procedure_performer_actor_reference_related_person",
        null=True,
    )
    device = models.ForeignKey(
        "devices.Device",
        on_delete=models.DO_NOTHING,
        related_name="procedure_performer_actor_reference_device",
        null=True,
    )
    care_team = models.ForeignKey(
        "careteams.CareTeam",
        on_delete=models.DO_NOTHING,
        related_name="procedure_performer_actor_reference_care_team",
        null=True,
    )
    healthcare_service = models.ForeignKey(
        "healthcareservices.HealthcareService",
        on_delete=models.DO_NOTHING,
        related_name="procedure_performer_actor_reference_healthcare_service",
        null=True,
    )


class ProcedureReasonReference(BaseReference):
    """Procedure Reason Reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        related_name="procedure_reason_reference_identifier",
        null=True,
    )
    condition = models.ForeignKey(
        "conditions.Condition",
        on_delete=models.DO_NOTHING,
        related_name="procedure_reason_reference_condition",
        null=True,
    )
    observation = models.ForeignKey(
        "observations.Observation",
        on_delete=models.DO_NOTHING,
        related_name="procedure_reason_reference_observation",
        null=True,
    )
    procedure = models.ForeignKey(
        "Procedure",
        on_delete=models.DO_NOTHING,
        related_name="procedure_reason_reference_procedure",
        null=True,
    )
    diagnostic_report = models.ForeignKey(
        "diagnosticreports.DiagnosticReport",
        on_delete=models.DO_NOTHING,
        related_name="procedure_reason_reference_diagnostic_report",
        null=True,
    )
    document_reference = models.ForeignKey(
        "documentreferences.DocumentReference",
        on_delete=models.DO_NOTHING,
        related_name="procedure_reason_reference_document_reference",
        null=True,
    )


class ProcedureReasonCodeableReference(TimeStampedModel):
    """Procedure Reason Codeable Reference model."""

    reference = models.ForeignKey(
        ProcedureReasonReference,
        on_delete=models.DO_NOTHING,
        related_name="procedure_reason_codeable_reference",
        null=True,
    )
    concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="procedure_reason_concept",
        null=True,
    )


class ProcedureUsedReference(BaseReference):
    """Procedure Used Reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        related_name="procedure_used_reference_identifier",
        null=True,
    )
    device = models.ForeignKey(
        "devices.Device",
        on_delete=models.DO_NOTHING,
        related_name="procedure_used_reference_device",
        null=True,
    )
    medication = models.ForeignKey(
        "medications.Medication",
        on_delete=models.DO_NOTHING,
        related_name="procedure_used_reference_medication",
        null=True,
    )
    substance = models.ForeignKey(
        "substances.Substance",
        on_delete=models.DO_NOTHING,
        related_name="procedure_used_reference_substance",
        null=True,
    )
    biologically_derived_product = models.ForeignKey(
        "biologicallyderivedproducts.BiologicallyDerivedProduct",
        on_delete=models.DO_NOTHING,
        related_name="procedure_used_reference_biologically_derived_product",
        null=True,
    )


class ProcedureUsedCodeableReference(TimeStampedModel):
    """Procedure Used Codeable Reference model."""

    reference = models.ForeignKey(
        ProcedureUsedReference,
        on_delete=models.DO_NOTHING,
        related_name="procedure_used_codeable_reference",
        null=True,
    )
    concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="procedure_used_concept",
        null=True,
    )


class ProcedureReportedReference(BaseReference):
    """Procedure Reported Reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        related_name="procedure_reported_reference_identifier",
        null=True,
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.DO_NOTHING,
        related_name="procedure_reported_reference_patient",
        null=True,
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.DO_NOTHING,
        related_name="procedure_reported_reference_related_person",
        null=True,
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.DO_NOTHING,
        related_name="procedure_reported_reference_practitioner",
        null=True,
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.DO_NOTHING,
        related_name="procedure_reported_reference_practitioner_role",
        null=True,
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.DO_NOTHING,
        related_name="procedure_reported_reference_organization",
        null=True,
    )


class ProcedurePerformer(TimeStampedModel):
    """Procedure Performer model."""

    function = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="procedure_performer_function",
        null=True,
    )
    actor = models.ForeignKey(
        ProcedurePerformerActorReference,
        on_delete=models.DO_NOTHING,
        related_name="procedure_performer_actor",
        null=True,
    )
    on_behalf_of = models.ForeignKey(
        OrganizationReference,
        on_delete=models.DO_NOTHING,
        related_name="procedure_performer_on_behalf_of",
        null=True,
    )
    period = models.ForeignKey(
        Period,
        on_delete=models.DO_NOTHING,
        related_name="procedure_performer_period",
        null=True,
    )


class ProcedureFocalDevice(TimeStampedModel):
    """Procedure Focal Device model."""

    action = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="procedure_focal_device_action",
        null=True,
    )
    manipulated = models.ForeignKey(
        "devices.DeviceReference",
        on_delete=models.DO_NOTHING,
        related_name="procedure_focal_device_manipulated",
        null=True,
    )


class ProcedureReportReference(BaseReference):
    """Procedure Report Reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        related_name="procedure_report_reference_identifier",
        null=True,
    )
    diagnostic_report = models.ForeignKey(
        "diagnosticreports.DiagnosticReport",
        on_delete=models.DO_NOTHING,
        related_name="procedure_reported_reference_diagnostic_report",
        null=True,
    )
    document_reference = models.ForeignKey(
        "documentreferences.DocumentReference",
        on_delete=models.DO_NOTHING,
        related_name="procedure_reported_reference_document_reference",
        null=True,
    )
    # composition = models.ForeignKey(
    #     "compositions.Composition",
    #     on_delete=models.DO_NOTHING,
    #     related_name="procedure_reported_reference_composition",
    #     null=True,
    # )


class Procedure(TimeStampedModel):
    """Procedure model."""

    identifier = models.ManyToManyField(
        Identifier,
        related_name="procedure_identifier",
        blank=True,
    )
    based_on = models.ManyToManyField(
        ProcedureBasedOnReference, blank=True, related_name="procedure_based_on"
    )
    part_of = models.ManyToManyField(
        ProcedurePartOfReference, blank=True, related_name="procedure_part_of"
    )
    status = models.CharField(
        max_length=255, choices=choices.ProcedureStatus.choices, null=True
    )
    status_reason = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="procedure_status_reason",
        null=True,
    )
    category = models.ManyToManyField(
        CodeableConcept, related_name="procedure_category", blank=True
    )
    code = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="procedure_code",
        null=True,
    )
    subject = models.ForeignKey(
        ProcedureSubjectReference,
        on_delete=models.DO_NOTHING,
        related_name="procedure_subject",
        null=True,
    )
    focus = models.ForeignKey(
        ProcedureFocusReference,
        on_delete=models.DO_NOTHING,
        related_name="procedure_focus",
        null=True,
    )
    encounter = models.ForeignKey(
        "encounters.EncounterReference",
        on_delete=models.DO_NOTHING,
        related_name="procedure_encounter",
        null=True,
    )
    occurrence_date_time = models.DateTimeField(null=True)
    occurrence_period = models.ForeignKey(
        Period,
        on_delete=models.DO_NOTHING,
        related_name="procedure_occurrence_period",
        null=True,
    )
    occurrence_string = models.CharField(max_length=255, null=True)
    occurrence_age = models.ForeignKey(
        Age,
        on_delete=models.DO_NOTHING,
        related_name="procedure_occurrence_age",
        null=True,
    )
    occurrence_range = models.ForeignKey(
        Range,
        on_delete=models.DO_NOTHING,
        related_name="procedure_occurrence_range",
        null=True,
    )
    occurrence_timing = models.ForeignKey(
        Timing,
        on_delete=models.DO_NOTHING,
        related_name="procedure_occurrence_timing",
        null=True,
    )
    recorded = models.DateTimeField(null=True)
    recorder = models.ForeignKey(
        ProcedureRecorderReference,
        on_delete=models.DO_NOTHING,
        related_name="procedure_recorder",
        null=True,
    )
    reported_boolean = models.BooleanField(null=True)
    reported_reference = models.ForeignKey(
        ProcedureReportedReference,
        on_delete=models.DO_NOTHING,
        related_name="procedure_reported_reference",
        null=True,
    )
    performer = models.ManyToManyField(
        ProcedurePerformer, blank=True, related_name="procedure_performer"
    )
    location = models.ForeignKey(
        "locations.LocationReference",
        on_delete=models.DO_NOTHING,
        related_name="procedure_location",
        null=True,
    )
    reason = models.ManyToManyField(
        ProcedureReasonCodeableReference, blank=True, related_name="procedure_reason"
    )
    body_site = models.ManyToManyField(
        CodeableConcept,
        related_name="procedure_body_site",
        blank=True,
    )
    body_structure = models.ManyToManyField(
        "bodystructures.BodyStructureReference",
        related_name="procedure_body_structure",
        blank=True,
    )
    outcome = models.ManyToManyField(
        "observations.ObservationCodeableReference",
        related_name="procedure_outcome",
        blank=True,
    )
    report = models.ManyToManyField(
        ProcedureReportReference,
        related_name="procedure_report",
        blank=True,
    )
    complication = models.ManyToManyField(
        "conditions.ConditionCodeableReference",
        related_name="procedure_complication",
        blank=True,
    )
    follow_up = models.ManyToManyField(
        "servicerequests.ServiceRequestPlanDefinitionReference",
        related_name="procedure_follow_up",
        blank=True,
    )
    note = models.ManyToManyField(
        Annotation,
        related_name="procedure_note",
        blank=True,
    )
    focal_device = models.ManyToManyField(
        ProcedureFocalDevice, blank=True, related_name="procedure_focal_device"
    )
    used = models.ManyToManyField(
        ProcedureUsedCodeableReference, blank=True, related_name="procedure_used"
    )
    supporting_info = models.ManyToManyField(
        Reference, blank=True, related_name="procedure_supporting_info"
    )


class ProcedureCodeableReference(TimeStampedModel):
    """Procedure Codeable Reference model."""

    reference = models.ForeignKey(
        ProcedureReference,
        on_delete=models.DO_NOTHING,
        related_name="procedure_codeable_reference_reference",
        null=True,
    )
    concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="procedure_codeable_reference_concept",
        null=True,
    )



"""observation definitions urls."""

from django.urls import path

from dfhir.observationdefinitions.views import (
    ObservationDefinitionDetailView,
    ObservationDefinitionListView,
)

app_name = "observationdefinitions"

urlpatterns = [
    path(
        "observationdefinitions/",
        ObservationDefinitionListView.as_view(),
        name="observationdefinitions",
    ),
    path(
        "observationdefinitions/<int:pk>/",
        ObservationDefinitionDetailView.as_view(),
        name="observationdefinition",
    ),
]



"""immunization recommendations views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from dfhir.immunizationrecommendations.models import ImmunizationRecommendation
from dfhir.immunizationrecommendations.serializers import (
    ImmunizationRecommendationSerializer,
)


class ImmunizationRecommendationListView(APIView):
    """immunization recommendation list view."""

    permission_classes = [AllowAny]

    @extend_schema(responses={200: ImmunizationRecommendationSerializer(many=True)})
    def get(self, request):
        """Get all objects."""
        queryset = ImmunizationRecommendation.objects.all()
        serializer = ImmunizationRecommendationSerializer(queryset, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        request=ImmunizationRecommendationSerializer,
        responses={201: ImmunizationRecommendationSerializer},
    )
    def post(self, request):
        """Create a new object."""
        serializer = ImmunizationRecommendationSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_201_CREATED)


class ImmunizationRecommendationDetailView(APIView):
    """immunization recommendation detail view."""

    def get_object(self, pk):
        """Get object by pk."""
        try:
            return ImmunizationRecommendation.objects.get(pk=pk)
        except ImmunizationRecommendation.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200: ImmunizationRecommendationSerializer})
    def get(self, request, pk):
        """Get object by pk."""
        obj = self.get_object(pk)
        serializer = ImmunizationRecommendationSerializer(obj)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        request=ImmunizationRecommendationSerializer,
        responses={200: ImmunizationRecommendationSerializer},
    )
    def patch(self, request, pk):
        """Update object by pk."""
        obj = self.get_object(pk)
        serializer = ImmunizationRecommendationSerializer(
            obj, data=request.data, partial=True
        )
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(responses={204: None})
    def delete(self, request, pk):
        """Delete object by pk."""
        obj = self.get_object(pk)
        obj.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



"""serializers file."""

from drf_writable_nested.serializers import WritableNestedModelSerializer
from rest_framework import serializers

from dfhir.base.serializers import (
    AnnotationSerializer,
    AttachmentSerializer,
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    IdentifierSerializer,
    PeriodSerializer,
    QuantitySerializer,
    RangeSerializer,
    RatioSerializer,
    ReferenceSerializer,
    TimingSerializer,
)
from dfhir.base.validators import validate_date_time_fields
from dfhir.bodystructures.serializers import BodyStructureReferenceSerializer
from dfhir.devices.serializers import DeviceDeviceMetricReferenceSerializer
from dfhir.encounters.serializers import EncounterReferenceSerializer
from dfhir.molecularsequences.serializers import MolecularSequenceReferenceSerializer
from dfhir.observationdefinitions.serializers import (
    ObservationDefinitionReferenceSerializer,
)
from dfhir.patients.serializers import PatientGroupReferenceSerializer

from .models import (
    DocumentReferenceObservationReference,
    Observation,
    ObservationBasedOnReference,
    ObservationBodyStructureReference,
    ObservationCodeableReference,
    ObservationComponent,
    ObservationDerivedFromReference,
    ObservationHasMemberReference,
    ObservationPartOfReference,
    ObservationPerformerReference,
    ObservationReference,
    ObservationReferenceRange,
    ObservationSubjectReference,
    ObservationTriggeredBy,
    SampledData,
    SampledDataIntervalUnitCodes,
)


class ObservationReferenceSerializer(BaseReferenceModelSerializer):
    """observation reference serializer."""

    identifier = IdentifierSerializer(required=False, many=False)

    class Meta:
        """meta options."""

        model = ObservationReference
        exclude = ["created_at", "updated_at"]


class ObservationBodyStructureReferenceSerializer(BaseReferenceModelSerializer):
    """observation body structure reference serializer."""

    identifier = IdentifierSerializer(required=False, many=False)

    class Meta:
        """meta options."""

        model = ObservationBodyStructureReference
        exclude = ["created_at", "updated_at"]


class ObservationBasedOnReferenceSerializer(WritableNestedModelSerializer):
    """observation based on reference serializer."""

    identifier = IdentifierSerializer(required=False, many=False)

    class Meta:
        """meta options."""

        model = ObservationBasedOnReference
        exclude = ["created_at", "updated_at"]


class ObservationPartOfReferenceSerializer(BaseReferenceModelSerializer):
    """observation part of reference serializer."""

    identifier = IdentifierSerializer(required=False, many=False)

    class Meta:
        """meta options."""

        model = ObservationPartOfReference
        exclude = ["created_at", "updated_at"]


class ObservationSubjectReferenceSerializer(BaseReferenceModelSerializer):
    """observation subject reference serializer."""

    identifier = IdentifierSerializer(required=False, many=False)

    class Meta:
        """meta options."""

        model = ObservationSubjectReference
        exclude = ["created_at", "updated_at"]


class ObservationTriggeredBySerializer(BaseReferenceModelSerializer):
    """observation triggered by serializer."""

    identifier = IdentifierSerializer(required=False, many=False)

    class Meta:
        """meta options."""

        model = ObservationTriggeredBy
        exclude = ["created_at", "updated_at"]


class ObservationHasMemberReferenceSerializer(BaseReferenceModelSerializer):
    """observation has member reference serializer."""

    identifier = IdentifierSerializer(required=False, many=False)

    class Meta:
        """meta options."""

        model = ObservationHasMemberReference
        exclude = ["created_at", "updated_at"]


class ObservationDerivedFromReferenceSerializer(BaseReferenceModelSerializer):
    """observation derived from reference serializer."""

    identifier = IdentifierSerializer(required=False, many=False)

    class Meta:
        """meta options."""

        model = ObservationDerivedFromReference
        exclude = ["created_at", "updated_at"]


class SampledDataIntervalUnitCodesSerializer(serializers.ModelSerializer):
    """sampled data interval unit codes serializer."""

    class Meta:
        """meta options."""

        model = SampledDataIntervalUnitCodes
        exclude = ["created_at", "updated_at"]


class SampledDataSerializer(WritableNestedModelSerializer):
    """sampled data serializer."""

    origin = QuantitySerializer(required=False, many=False)
    interval_unit = SampledDataIntervalUnitCodesSerializer(required=False, many=False)

    class Meta:
        """meta options."""

        model = SampledData
        exclude = ["created_at", "updated_at"]


class ObservationPerformerReferenceSerializer(BaseReferenceModelSerializer):
    """observation performer reference serializer."""

    identifier = IdentifierSerializer(required=False, many=False)

    class Meta:
        """meta options."""

        model = ObservationPerformerReference
        exclude = ["created_at", "updated_at"]


class ObservationReferenceRangeSerializer(WritableNestedModelSerializer):
    """observation reference range serializer."""

    low = QuantitySerializer(required=False, many=False)
    high = QuantitySerializer(required=False, many=False)
    normal_value = CodeableConceptSerializer(required=False, many=False)
    type = CodeableConceptSerializer(required=False, many=False)
    applies_to = CodeableConceptSerializer(required=False, many=True)
    age = RangeSerializer(required=False, many=True)

    class Meta:
        """meta options."""

        model = ObservationReferenceRange
        exclude = ["created_at", "updated_at"]


class ObservationComponentSerializer(WritableNestedModelSerializer):
    """observation component serializer."""

    code = CodeableConceptSerializer(required=False, many=False)
    value_quantity = QuantitySerializer(required=False, many=False)
    value_codeable_concept = CodeableConceptSerializer(required=False, many=False)
    value_range = RangeSerializer(required=False, many=False)
    value_ratio = RatioSerializer(required=False, many=False)
    value_sampled_data = SampledDataSerializer(required=False, many=False)
    value_period = PeriodSerializer(required=False, many=False)
    value_attachment = AttachmentSerializer(required=False, many=False)
    value_reference = MolecularSequenceReferenceSerializer(required=False, many=False)
    data_absent_reason = CodeableConceptSerializer(required=False, many=False)
    interpretation = CodeableConceptSerializer(required=False, many=True)
    reference_range = ObservationReferenceRangeSerializer(required=False, many=True)

    class Meta:
        """meta options."""

        model = ObservationComponent
        exclude = ["created_at", "updated_at"]


class ObservationSerializer(BaseWritableNestedModelSerializer):
    """observation serializer."""

    identifier = IdentifierSerializer(required=False, many=True)
    # TODO: instantiates_canonical = ObservationDefinitionReferenceSerializer()
    instantiates_reference = ObservationDefinitionReferenceSerializer(
        required=False, many=True
    )
    based_on = ObservationBasedOnReferenceSerializer(required=False, many=True)
    triggered_by = ObservationTriggeredBySerializer(required=False, many=True)
    part_of = ObservationPartOfReferenceSerializer(required=False, many=True)
    category = CodeableConceptSerializer(required=False, many=True)
    code = CodeableConceptSerializer(required=False, many=False)
    subject = ObservationSubjectReferenceSerializer(required=False, many=False)
    focus = ReferenceSerializer(required=False, many=True)
    encounter = EncounterReferenceSerializer(required=False, many=False)
    effective_period = PeriodSerializer(required=False, many=False)
    effective_timing = TimingSerializer(required=False, many=False)
    performer = ObservationPerformerReferenceSerializer(required=False, many=True)
    value_quantity = QuantitySerializer(required=False, many=False)
    value_codeable_concept = CodeableConceptSerializer(required=False, many=False)
    value_range = RangeSerializer(required=False, many=False)
    value_ratio = RatioSerializer(required=False, many=False)
    value_sampled_data = SampledDataSerializer(required=False, many=False)
    value_period = PeriodSerializer(required=False, many=False)
    value_attachment = AttachmentSerializer(required=False, many=False)
    value_reference = MolecularSequenceReferenceSerializer(required=False, many=False)
    data_absent_reason = CodeableConceptSerializer(required=False, many=False)
    interpretation = CodeableConceptSerializer(required=False, many=True)
    note = AnnotationSerializer(required=False, many=True)
    body_site = CodeableConceptSerializer(required=False, many=False)
    body_structure = BodyStructureReferenceSerializer(required=False, many=False)
    method = CodeableConceptSerializer(required=False, many=False)
    specimen = PatientGroupReferenceSerializer(required=False, many=False)
    device = DeviceDeviceMetricReferenceSerializer(required=False, many=False)
    reference_range = ObservationReferenceRangeSerializer(required=False, many=True)
    has_member = ObservationHasMemberReferenceSerializer(required=False, many=True)
    derived_from = ObservationDerivedFromReferenceSerializer(required=False, many=True)
    component = ObservationComponentSerializer(required=False, many=True)

    class Meta:
        """meta options."""

        model = Observation
        exclude = ["created_at", "updated_at"]

    def validate(self, data):
        """Validate serializer data."""
        validate_date_time_fields(
            data.get("effective_start_datetime"), data.get("effective_end_datetime")
        )
        return data


class ObservationCodeableReferenceSerializer(WritableNestedModelSerializer):
    """observation codeable reference serializer."""

    concept = CodeableConceptSerializer(required=False, many=False)
    reference = ObservationReferenceSerializer(required=False, many=False)

    class Meta:
        """meta options."""

        model = ObservationCodeableReference
        exclude = ["created_at", "updated_at"]


class DocumentReferenceObservationReferenceSerializer(BaseReferenceModelSerializer):
    """Document Reference Observation Reference Serializer."""

    identifier = IdentifierSerializer(required=False, many=False)

    class Meta:
        """Meta class."""

        model = DocumentReferenceObservationReference
        exclude = ["created_at", "updated_at"]



# Generated by Django 5.2 on 2025-04-23 12:46

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('appointmentresponses', '0002_initial'),
        ('base', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='appointmentresponse',
            name='identifier',
            field=models.ManyToManyField(blank=True, related_name='appointment_response_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='appointmentresponse',
            name='participant_type',
            field=models.ManyToManyField(blank=True, related_name='appointment_response_participant_type', to='base.codeableconcept'),
        ),
    ]



"""Personal Relationship Target Reference urls."""

from django.urls import path

from . import views

app_name = "personalrelationships"

urlpatterns = [
    path(
        "personalrelationships/",
        views.PersonalRelationshipListView.as_view(),
    ),
    path(
        "personalrelationships/<int:pk>/",
        views.PersonalRelationshipDetailView.as_view(),
    ),
]



"""molecular sequences model choices."""

from django.db.models import TextChoices
from django.utils.translation import gettext_lazy as _


class MolecularSequenceTypeChoices(TextChoices):
    """Molecular sequence type choices."""

    AA = "aa", _("Amino acid")
    DNA = "dna", _("DNA")
    RNA = "rna", _("RNA")



# Generated by Django 5.2 on 2025-04-23 12:46

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='AdverseEvent',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('status', models.CharField(choices=[('in-progress', 'in-progress'), ('completed', 'completed'), ('entered-in-error', 'entered-in-error'), ('unknown', 'unknown')], max_length=255)),
                ('actuality', models.CharField(choices=[('actual', 'actual'), ('potential', 'potential')], max_length=255)),
                ('cause_date_time', models.DateTimeField(null=True)),
                ('effect_date_time', models.DateTimeField(null=True)),
                ('detected', models.DateTimeField(null=True)),
                ('recorded_date', models.DateTimeField(null=True)),
                ('expected_in_research_study', models.BooleanField(default=False)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='AdverseEventContributingFactorCodeableReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='AdverseEventContributingFactorReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='AdverseEventMitigatingActionCodeableReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='AdverseEventMitigatingActionReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='AdverseEventParticipant',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='AdverseEventParticipantActorReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='AdverseEventPreventiveActionCodeableReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='AdverseEventPreventiveActionReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='AdverseEventRecorderReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='AdverseEventSubjectReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='AdverseEventSupportingInfoCodeableReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='AdverseEventSupportingInfoReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='AdverseEventSuspectEntity',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='AdverseEventSuspectEntityAuthorReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='AdverseEventSuspectEntityCausality',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='AdverseEventSuspectEntityInstanceCodeableReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='AdverseEventSuspectEntityInstanceReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



"""Group choices."""

from django.db import models
from django.utils.translation import gettext_lazy as _


class GroupStatus(models.TextChoices):
    """Group status choices."""

    DRAFT = "draft", _("Draft")
    ACTIVE = "active", _("Active")
    RETIRED = "retired", _("Retired")
    UNKNOWN = "unknown", _("Unknown")


class GroupMembershipChoices(models.TextChoices):
    """Group membership choices."""

    DEFINITIONAL = "definitional", _("Definitional")
    CONCEPTUAL = "conceptual", _("Conceptual")
    ENUMERATED = "enumerated", _("Enumerated")


class GroupCombinationMethodChoices(models.TextChoices):
    """Group combination method choices."""

    ALL_OF = "all-of", _("All Of")
    ANY_OF = "any-of", _("Any Of")
    AT_LEAST = "at-least", _("At Least")
    AT_MOST = "at-most", _("At Most")
    EXCEPT_SUBSET = "except-subset", _("Except Subset")


class GroupTypeChoices(models.TextChoices):
    """Group types choices."""

    PERSON = "person", _("Person")
    ANIMAL = "animal", _("Animal")
    PRACTITIONER = "practitioner", _("Practitioner")
    DEVICE = "device", _("Device")
    CARE_TEAM = "care-team", _("Care Team")
    HEALTHCARE_SERVICE = "healthcare-service", _("Healthcare Service")
    LOCATION = "location", _("Location")
    ORGANIZATION = "organization", _("Organization")



"""Model for EpisodeOfCare."""

from django.db import models

from dfhir.base.models import (
    BaseReference,
    CodeableConcept,
    Identifier,
    OrganizationReference,
    Period,
    TimeStampedModel,
)

from . import choices


class EpisodeOfCareStatusHistory(TimeStampedModel):
    """Model for EpisodeOfCareStatusHistory."""

    status = models.CharField(
        max_length=255, choices=choices.EpisodeOfCareStatusChoices.choices, null=True
    )
    period = models.ForeignKey(
        Period,
        related_name="episode_of_care_status_history_period",
        on_delete=models.CASCADE,
        null=True,
    )


class EpisodeOfCareReasonValueReference(BaseReference):
    """Reference to an EpisodeOfCareReasonValue."""

    identifier = models.ForeignKey(
        Identifier,
        related_name="episode_of_care_reason_value_reference_identifier",
        on_delete=models.CASCADE,
        null=True,
    )
    condition = models.ForeignKey(
        "conditions.Condition",
        related_name="episode_of_care_reason_value_reference_condition",
        on_delete=models.CASCADE,
        null=True,
    )
    procedure = models.ForeignKey(
        "procedures.Procedure",
        related_name="episode_of_care_reason_value_reference_procedure",
        on_delete=models.CASCADE,
        null=True,
    )
    observation = models.ForeignKey(
        "observations.Observation",
        related_name="episode_of_care_reason_value_reference_observation",
        on_delete=models.CASCADE,
        null=True,
    )
    healthcare_service = models.ForeignKey(
        "healthcareservices.HealthCareService",
        related_name="episode_of_care_reason_value_reference_healthcare_service",
        on_delete=models.CASCADE,
        null=True,
    )


class EpisodeOfCareReasonValueCodeableReference(TimeStampedModel):
    """Model for EpisodeOfCareReasonValueCodeableReference."""

    reference = models.ForeignKey(
        EpisodeOfCareReasonValueReference,
        related_name="episode_of_care_reason_value_codeable_reference_reference",
        on_delete=models.CASCADE,
        null=True,
    )
    concept = models.ForeignKey(
        CodeableConcept,
        related_name="episode_of_care_reason_value_codeable_reference_concept",
        on_delete=models.CASCADE,
        null=True,
    )


class EpisodeOfCareReason(TimeStampedModel):
    """Model for EpisodeOfCareReason."""

    use = models.ForeignKey(
        CodeableConcept,
        related_name="episode_of_care_reason_use",
        on_delete=models.CASCADE,
        null=True,
    )
    value = models.ForeignKey(
        EpisodeOfCareReasonValueCodeableReference,
        related_name="episode_of_care_reason_value",
        on_delete=models.CASCADE,
        null=True,
    )


class EpisodeOfCareDiagnosis(TimeStampedModel):
    """Model for EpisodeOfCareDiagnosis."""

    condition = models.ForeignKey(
        "conditions.ConditionCodeableReference",
        related_name="episode_of_care_diagnosis_condition",
        on_delete=models.CASCADE,
        null=True,
    )
    use = models.ForeignKey(
        CodeableConcept,
        related_name="episode_of_care_diagnosis_use",
        on_delete=models.CASCADE,
        null=True,
    )


class EpisodeOfCare(TimeStampedModel):
    """Model for EpisodeOfCare."""

    identifier = models.ManyToManyField(
        Identifier, related_name="episode_of_care_identifier", blank=True
    )
    status = models.CharField(
        max_length=255, choices=choices.EpisodeOfCareStatusChoices.choices, null=True
    )
    status_history = models.ManyToManyField(
        EpisodeOfCareStatusHistory,
        related_name="episode_of_care_status_history",
        blank=True,
    )
    type = models.ManyToManyField(
        CodeableConcept, related_name="episode_of_care_type", blank=True
    )
    diagnosis = models.ManyToManyField(
        EpisodeOfCareDiagnosis, related_name="episode_of_care_diagnosis", blank=True
    )
    subject = models.ForeignKey(
        "patients.PatientGroupReference",
        related_name="episode_of_care_patient",
        on_delete=models.CASCADE,
    )
    managing_organization = models.ForeignKey(
        OrganizationReference,
        related_name="episode_of_care_managing_organization",
        on_delete=models.CASCADE,
    )
    period = models.ForeignKey(
        Period,
        related_name="episode_of_care_period",
        on_delete=models.CASCADE,
        null=True,
    )
    referral_request = models.ManyToManyField(
        "servicerequests.ServiceRequestReference",
        related_name="episode_of_care_referral_request",
        blank=True,
    )
    care_manager = models.ForeignKey(
        "practitioners.PractitionerPractitionerRoleReference",
        related_name="episode_of_care_care_manager",
        on_delete=models.CASCADE,
        null=True,
    )
    care_team = models.ManyToManyField(
        "careteams.CareTeamReference",
        related_name="episode_of_care_care_team",
        blank=True,
    )
    account = models.ManyToManyField(
        "accounts.AccountReference", related_name="episode_of_care_account", blank=True
    )


class EpisodeOfCareReference(BaseReference):
    """Reference to an EpisodeOfCare."""

    identifier = models.ForeignKey(
        Identifier,
        related_name="episode_of_care_reference_identifier",
        on_delete=models.CASCADE,
        null=True,
    )
    episode_of_care = models.ForeignKey(
        EpisodeOfCare,
        related_name="episode_of_care_reference_episode_of_care",
        on_delete=models.CASCADE,
        null=True,
    )



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0001_initial'),
        ('substances', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='NutritionProductCharacteristic',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('value_string', models.CharField(max_length=255, null=True)),
                ('value_boolean', models.BooleanField(null=True)),
                ('value_base64_binary', models.TextField(null=True)),
                ('type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_product_characteristic_type', to='base.codeableconcept')),
                ('value_attachment', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_product_characteristic_value_attachment', to='base.attachment')),
                ('value_codeable_concept', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_product_characteristic_value_codeable_concept', to='base.codeableconcept')),
                ('value_quantity', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_product_characteristic_value_quantity', to='base.quantity')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='NutritionProductCodeableReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('concept', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_product_codeable_reference_concept', to='base.codeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='NutritionProductIngredient',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('allergen', models.BooleanField(null=True)),
                ('amount_quantity', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_product_ingredient_amount_quantity', to='base.quantity')),
                ('amount_ratio', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_product_ingredient_amount_ratio', to='base.ratio')),
                ('item', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_product_ingredient_item', to='nutritionproducts.nutritionproductcodeablereference')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='NutritionProductInstance',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('name', models.CharField(max_length=255, null=True)),
                ('lot_number', models.CharField(max_length=255, null=True)),
                ('expiry', models.DateTimeField(null=True)),
                ('use_by', models.DateTimeField(null=True)),
                ('biological_source_event', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_product_instance_biological_source_event', to='base.identifier')),
                ('identifier', models.ManyToManyField(blank=True, related_name='nutrition_product_instance_identifier', to='base.identifier')),
                ('quantity', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_product_instance_quantity', to='base.quantity')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='NutritionProductNutrients',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('amount_quantity', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_product_nutrients_amount_quantity', to='base.quantity')),
                ('amount_ratio', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_product_nutrients_amount_ratio', to='base.ratio')),
                ('item', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_product_nutrients_item', to='substances.substancecodeablereference')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='NutritionProduct',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('status', models.CharField(choices=[('active', 'Active'), ('inactive', 'Inactive'), ('entered-in-error', 'Entered in Error')], max_length=255, null=True)),
                ('ingredient_summary', models.TextField(null=True)),
                ('category', models.ManyToManyField(blank=True, related_name='nutrition_product_category', to='base.codeableconcept')),
                ('code', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_product_code', to='base.codeableconcept')),
                ('energy', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_product_energy', to='base.quantity')),
                ('manufacturer', models.ManyToManyField(blank=True, related_name='nutrition_product_manufacturer', to='base.organizationreference')),
                ('note', models.ManyToManyField(blank=True, related_name='nutrition_product_note', to='base.annotation')),
                ('characteristic', models.ManyToManyField(blank=True, related_name='nutrition_product_characteristic', to='nutritionproducts.nutritionproductcharacteristic')),
                ('ingredient', models.ManyToManyField(blank=True, related_name='nutrition_product_ingredient', to='nutritionproducts.nutritionproductingredient')),
                ('instance', models.ManyToManyField(blank=True, related_name='nutrition_product_instance', to='nutritionproducts.nutritionproductinstance')),
                ('nutrient', models.ManyToManyField(blank=True, related_name='nutrition_product_nutrient', to='nutritionproducts.nutritionproductnutrients')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='NutritionProductReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_product_reference_identifier', to='base.identifier')),
                ('nutrition_product', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_product_reference_product', to='nutritionproducts.nutritionproduct')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='nutritionproductcodeablereference',
            name='reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_product_codeable_reference_reference', to='nutritionproducts.nutritionproductreference'),
        ),
    ]



"""molecular definitions models."""

from django.contrib.postgres.fields import ArrayField
from django.db import models

from dfhir.base.models import (
    BaseReference,
    CodeableConcept,
    Identifier,
    TimeStampedModel,
)

# Create your models here.


class MolecularDefinitionReference(BaseReference):
    """Molecular definition reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="molecular_definition_reference_identifier",
    )
    molecular_definition = models.ForeignKey(
        "MolecularDefinition",
        on_delete=models.DO_NOTHING,
        related_name="molecular_definition_reference_molecular_definition",
        null=True,
    )


class MolecularDefinitionCoordinateIntervalCoordinateSystem(TimeStampedModel):
    """molecular definition coordinate interval coordinate system model."""

    system = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="molecular_definition_coordinate_interval_coordinate_system_system",
    )
    origin = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="molecular_definition_coordinate_interval_coordinate_system_origin",
    )
    normalization_method = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="molecular_definition_coordinate_interval_coordinate_system_method",
    )

    start_quantity = models.ForeignKey(
        "base.Quantity",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="molecular_definition_coordinate_interval_coordinate_system_start_quantity",
    )
    start_range = models.ForeignKey(
        "base.Range",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="molecular_definition_coordinate_interval_coordinate_system_start_range",
    )
    end_quantity = models.ForeignKey(
        "base.Quantity",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="molecular_definition_coordinate_interval_coordinate_system_end_quantity",
    )
    end_range = models.ForeignKey(
        "base.Range",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="molecular_definition_coordinate_interval_coordinate_system_end_range",
    )


class MolecularDefinitionCoordinateInterval(TimeStampedModel):
    """molecular definition relative edit coordinate interval model."""

    coordinate_system = models.ForeignKey(
        MolecularDefinitionCoordinateIntervalCoordinateSystem,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="molecular_definition_relative_edit_coordinate_system",
    )


class MolecularDefinitionSequenceLocation(TimeStampedModel):
    """Sequence location model."""

    sequence_context = models.ForeignKey(
        MolecularDefinitionReference,
        on_delete=models.DO_NOTHING,
        blank=True,
        related_name="sequence_location_sequence_context",
    )
    coordinated_interval = models.ForeignKey(
        MolecularDefinitionCoordinateInterval,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="sequence_location_coordinated_internal",
    )
    strand = models.ForeignKey(
        CodeableConcept,
        null=True,
        on_delete=models.DO_NOTHING,
        related_name="sequence_location_strand",
    )


class MolecularSequenceGenomeAssembly(TimeStampedModel):
    """Genome assembly model."""

    organism = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="genome_assembly_organism",
    )
    build = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="genome_assembly_build",
    )
    accession = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="genome_assembly_accession",
    )
    description_markdown = models.TextField(null=True)
    description_string = models.CharField(max_length=255, null=True)


class MolecularDefinitionCytobandIntervalStartEndCytoband(TimeStampedModel):
    """Cytoband interval start cytoband model."""

    arm_code = models.CharField(max_length=255, null=True)
    arm_string = models.CharField(max_length=255, null=True)
    region_code = models.CharField(max_length=255, null=True)
    region_string = models.CharField(max_length=255, null=True)
    band_code = models.CharField(max_length=255, null=True)
    band_string = models.CharField(max_length=255, null=True)
    sub_band_code = models.CharField(max_length=255, null=True)
    sub_band_string = models.CharField(max_length=255, null=True)


class MolecularDefinitionCytobandInterval(TimeStampedModel):
    """Cytoband interval model."""

    chromosome = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="cytoband_interval_chromosome",
    )
    start_cytoband = models.ForeignKey(
        MolecularDefinitionCytobandIntervalStartEndCytoband,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="cytoband_interval_start_cytoband",
    )
    end_cytoband = models.ForeignKey(
        MolecularDefinitionCytobandIntervalStartEndCytoband,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="cytoband_interval_end_cytoband",
    )


class MolecularDefinitionCytobandLocation(TimeStampedModel):
    """Cytoband location model."""

    genome_assembly = models.ForeignKey(
        MolecularSequenceGenomeAssembly,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="cytoband_location_genome_assembly",
    )
    cytoband_interval = models.ForeignKey(
        MolecularDefinitionCytobandInterval,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="cytoband_location_cytoband_interval",
    )


class MolecularDefinitionLocation(TimeStampedModel):
    """Molecular definition location model."""

    sequence_location = models.ForeignKey(
        MolecularDefinitionSequenceLocation,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="molecular_definition_sequence_location",
    )
    cytoband_location = models.ForeignKey(
        MolecularDefinitionCytobandLocation,
        on_delete=models.DO_NOTHING,
        related_name="molecular_definition_cytoband_location",
        null=True,
    )
    featured_location = ArrayField(
        models.CharField(max_length=255, null=True), null=True
    )


class MolecularDefinitionRepresentationLiteral(TimeStampedModel):
    """representation literal model."""

    encoding = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="representation_literal_encoding",
    )
    value = models.CharField(max_length=255, null=True)


class MolecularDefinitionRepresentationExtracted(TimeStampedModel):
    """representation extracted model."""

    starting_molecule = models.ForeignKey(
        MolecularDefinitionReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="representation_extract_string_molecule",
    )
    coordinate_interval = models.ForeignKey(
        MolecularDefinitionCoordinateInterval,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="molecular_definition_extracted_coordinate_interval",
    )
    reverse_complement = models.BooleanField(default=False)


class MolecularDefinitionRepresentationRepeated(TimeStampedModel):
    """representation repeated model."""

    sequence_motif = models.ForeignKey(
        MolecularDefinitionReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="representation_extract_sequence_motif",
    )
    copy_count = models.IntegerField(null=True)


class MolecularDefinitionRepresentationConcatenatedSequenceElement(TimeStampedModel):
    """representation concatenated sequence element model."""

    sequence = models.ForeignKey(
        MolecularDefinitionReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="representation_concatenated_sequence_element_sequence",
    )
    ordinal_index = models.IntegerField(null=True)


class MolecularDefinitionRepresentationConcatenated(TimeStampedModel):
    """representation concatenated model."""

    sequence_element = models.ManyToManyField(
        MolecularDefinitionRepresentationConcatenatedSequenceElement,
        blank=True,
        related_name="representation_concatenated",
    )


class MolecularDefinitionRepresentationRelativeEdit(TimeStampedModel):
    """representation relative edit model."""

    edit_order = models.IntegerField(null=True)
    coordinate_interval = models.ForeignKey(
        MolecularDefinitionCoordinateInterval,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="molecular_representation_relative_edit_coordinate_interval",
    )
    coordinate_system = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="molecular_representation_coordinate_system",
        null=True,
    )
    replacement_molecule = models.ForeignKey(
        MolecularDefinitionReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="molecular_representation_relative_edit_replacement_molecule",
    )
    replaced_molecule = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="molecular_representation_edit_replaced_molecule",
        null=True,
    )


class MolecularDefinitionRepresentationRelative(TimeStampedModel):
    """representation relative model."""

    starting_molecule = models.ForeignKey(
        MolecularDefinitionReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="representation_starting_molecule",
    )
    edit = models.ManyToManyField(
        MolecularDefinitionRepresentationRelativeEdit,
        related_name="representation_edit_relative",
        blank=True,
    )


class MolecularDefinitionRepresentation(TimeStampedModel):
    """representation model."""

    focus = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="representation_focus",
    )
    code = models.ManyToManyField(
        CodeableConcept, blank=True, related_name="representation_code"
    )
    literal = models.ForeignKey(
        MolecularDefinitionRepresentationLiteral,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="representation_literal",
    )
    resolvable = models.ForeignKey(
        "documentreferences.DocumentReferenceReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="representation_resolvable",
    )
    extracted = models.ForeignKey(
        MolecularDefinitionRepresentationExtracted,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="representation_extracted",
    )
    repeated = models.ForeignKey(
        MolecularDefinitionRepresentationRepeated,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="representation_repeat",
    )
    concatenated = models.ForeignKey(
        MolecularDefinitionRepresentationConcatenated,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="representation_concatenated",
    )
    relative = models.ForeignKey(
        MolecularDefinitionRepresentationRelative,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="representation_relative",
    )


class MolecularDefinition(TimeStampedModel):
    """molecular definition model."""

    identifier = models.ManyToManyField(
        Identifier, blank=True, related_name="molecular_definition_identifier"
    )
    description = models.TextField(null=True)
    molecular_type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="molecular_definition_molecular_type",
    )
    type = models.ManyToManyField(
        CodeableConcept, blank=True, related_name="molecular_definition_type"
    )
    topology = models.ManyToManyField(
        CodeableConcept, blank=True, related_name="molecular_definition_topology"
    )
    member_state = models.ManyToManyField(
        MolecularDefinitionReference,
        blank=True,
        related_name="molecular_definition_member_state",
    )
    location = models.ManyToManyField(
        MolecularDefinitionLocation,
        blank=True,
        related_name="molecular_definition_location",
    )
    representation = models.ManyToManyField(
        MolecularDefinitionRepresentation,
        blank=True,
        related_name="molecular_definition_representation",
    )



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('accounts', '0003_initial'),
        ('base', '0005_initial'),
        ('devices', '0001_initial'),
        ('healthcareservices', '0001_initial'),
        ('locations', '0001_initial'),
        ('organizations', '0001_initial'),
        ('patients', '0002_initial'),
        ('practitionerroles', '0002_initial'),
        ('practitioners', '0001_initial'),
        ('procedures', '0001_initial'),
        ('relatedpersons', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='accountprocedure',
            name='code',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='account_procedure_code', to='procedures.procedurecodeablereference'),
        ),
        migrations.AddField(
            model_name='accountprocedure',
            name='device',
            field=models.ManyToManyField(blank=True, related_name='account_procedure_device', to='devices.devicereference'),
        ),
        migrations.AddField(
            model_name='accountprocedure',
            name='package_code',
            field=models.ManyToManyField(blank=True, related_name='account_procedure_package_code', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='accountprocedure',
            name='type',
            field=models.ManyToManyField(blank=True, related_name='account_procedure_type', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='account',
            name='procedure',
            field=models.ManyToManyField(blank=True, related_name='accounts_procedure', to='accounts.accountprocedure'),
        ),
        migrations.AddField(
            model_name='accountreference',
            name='account',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='account_reference_account', to='accounts.account'),
        ),
        migrations.AddField(
            model_name='accountreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='account_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='accountrelatedaccount',
            name='account',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='account_related_account_reference', to='accounts.accountreference'),
        ),
        migrations.AddField(
            model_name='accountrelatedaccount',
            name='relationship',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='account_related_account_relationship', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='account',
            name='related_account',
            field=models.ManyToManyField(blank=True, related_name='accounts_related_account', to='accounts.accountrelatedaccount'),
        ),
        migrations.AddField(
            model_name='accountsguarantorpartyreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='accounts_guarantor_party_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='accountsguarantorpartyreference',
            name='organization',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='accounts_guarantor_party_reference_organization', to='organizations.organization'),
        ),
        migrations.AddField(
            model_name='accountsguarantorpartyreference',
            name='patient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='accounts_guarantor_party_reference_patient', to='patients.patient'),
        ),
        migrations.AddField(
            model_name='accountsguarantorpartyreference',
            name='related_person',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='accounts_guarantor_party_reference_related_person', to='relatedpersons.relatedperson'),
        ),
        migrations.AddField(
            model_name='accountguarantor',
            name='party',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='account_guarantor_party', to='accounts.accountsguarantorpartyreference'),
        ),
        migrations.AddField(
            model_name='accountsubjectreference',
            name='device',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='account_subject_reference_device', to='devices.device'),
        ),
        migrations.AddField(
            model_name='accountsubjectreference',
            name='healthcare_service',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='account_subject_reference_healthcare_service', to='healthcareservices.healthcareservice'),
        ),
        migrations.AddField(
            model_name='accountsubjectreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='account_subject_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='accountsubjectreference',
            name='location',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='account_subject_reference_location', to='locations.location'),
        ),
        migrations.AddField(
            model_name='accountsubjectreference',
            name='organization',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='account_subject_reference_organization', to='organizations.organization'),
        ),
        migrations.AddField(
            model_name='accountsubjectreference',
            name='patient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='account_subject_reference_patient', to='patients.patient'),
        ),
        migrations.AddField(
            model_name='accountsubjectreference',
            name='practitioner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='account_subject_reference_practitioner', to='practitioners.practitioner'),
        ),
        migrations.AddField(
            model_name='accountsubjectreference',
            name='practitioner_role',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='account_subject_reference_practitioner_role', to='practitionerroles.practitionerrole'),
        ),
        migrations.AddField(
            model_name='account',
            name='subject',
            field=models.ManyToManyField(blank=True, related_name='accounts_subject', to='accounts.accountsubjectreference'),
        ),
    ]



"""specimen model choices."""

from django.db.models import TextChoices
from django.utils.translation import gettext_lazy as _


class SpecimenStatus(TextChoices):
    """specimen status choices."""

    AVAILABLE = "available", _("Available")
    UNAVAILABLE = "unavailable", _("Unavailable")
    UNSATISFACTORY = "unsatisfactory", _("Unsatisfactory")
    ENTERED_IN_ERROR = "entered-in-error", _("Entered in Error")


class SpecimenCombinedChoices(TextChoices):
    """specimen combined choices."""

    GROUPED = "grouped", _("Grouped")
    POOLED = "pooled", _("Pooled")



"""Deviceassociations serializers."""

from drf_writable_nested import WritableNestedModelSerializer

from dfhir.base.serializers import (
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    IdentifierSerializer,
    PeriodSerializer,
)
from dfhir.bodystructures.serializers import BodyStructureReferenceSerializer
from dfhir.devices.serializers import DeviceReferenceSerializer

from .models import (
    DeviceAssociation,
    DeviceAssociationOperation,
    DeviceAssociationOperationOperatorReference,
    DeviceAssociationSubjectReference,
)


class DeviceAssociationSubjectReferenceSerializer(BaseReferenceModelSerializer):
    """Device Association Subject Reference Serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = DeviceAssociationSubjectReference
        exclude = ["created_at", "updated_at"]


class DeviceAssociationOperationOperatorReferenceSerializer(
    BaseReferenceModelSerializer
):
    """Device Association Operation Operator Reference Serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = DeviceAssociationOperationOperatorReference
        exclude = ["created_at", "updated_at"]


class DeviceAssociationOperationSerializer(WritableNestedModelSerializer):
    """Device Association Operation Serializer."""

    status = CodeableConceptSerializer(many=False, required=False)
    operator = DeviceAssociationOperationOperatorReferenceSerializer(
        many=True, required=False
    )
    period = PeriodSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = DeviceAssociationOperation
        exclude = ["created_at", "updated_at"]


class DeviceAssociationSerializer(BaseWritableNestedModelSerializer):
    """Device Association Serializer."""

    identifier = IdentifierSerializer(many=True, required=False)
    device = DeviceReferenceSerializer(many=False, required=False)
    relationship = CodeableConceptSerializer(many=False, required=False)
    status = CodeableConceptSerializer(many=False, required=False)
    status_reason = CodeableConceptSerializer(many=False, required=False)
    subject = DeviceAssociationSubjectReferenceSerializer(many=False, required=False)
    body_structure = BodyStructureReferenceSerializer(many=False, required=False)
    period = PeriodSerializer(many=False, required=False)
    operation = DeviceAssociationOperationSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = DeviceAssociation
        exclude = ["created_at", "updated_at"]



"""image selection serializers."""

from drf_writable_nested.serializers import WritableNestedModelSerializer
from rest_framework import serializers

from dfhir.base.serializers import (
    BaseReferenceModelSerializer,
    CodeableConceptSerializer,
    IdentifierSerializer,
)
from dfhir.endpoints.serializers import EndpointReferenceSerializer
from dfhir.imagingselections.models import (
    ImagingSelection,
    ImagingSelectionBasedOnReference,
    ImagingSelectionCodeableReference,
    ImagingSelectionImageRegion3D,
    ImagingSelectionInstance,
    ImagingSelectionInstanceImagingRegion2D,
    ImagingSelectionPerformer,
    ImagingSelectionPerformerActorReference,
    ImagingSelectionReference,
    ImagingSelectionSubjectReference,
)

# from dfhir.imagingstudies.serializers import ImagingStudyReferenceSerializer


class ImagingSelectionPerformerActorReferenceSerializer(BaseReferenceModelSerializer):
    """image selection performer actor reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """meta options."""

        model = ImagingSelectionPerformerActorReference
        exclude = ["created_at", "updated_at"]


class ImagingSelectionPerformerSerializer(WritableNestedModelSerializer):
    """image selection performer serializer."""

    function = CodeableConceptSerializer(required=False)
    actor = ImagingSelectionPerformerActorReferenceSerializer(required=False)

    class Meta:
        """meta options."""

        model = ImagingSelectionPerformer
        exclude = ["created_at", "updated_at"]


class ImagingSelectionBasedOnReferenceSerializer(BaseReferenceModelSerializer):
    """image selection based on reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """meta options."""

        model = ImagingSelectionBasedOnReference
        exclude = ["created_at", "updated_at"]


class ImagingSelectionInstanceImagingRegion2DSerializer(serializers.ModelSerializer):
    """image selection instance imaging region 2d serializer."""

    class Meta:
        """meta options."""

        model = ImagingSelectionInstanceImagingRegion2D
        exclude = ["created_at", "updated_at"]


class ImagingSelectionInstanceSerializer(WritableNestedModelSerializer):
    """imaging selection instance serializer."""

    imaging_region = ImagingSelectionInstanceImagingRegion2DSerializer(
        many=True, required=False
    )

    class Meta:
        """meta options."""

        model = ImagingSelectionInstance
        exclude = ["created_at", "updated_at"]


class ImagingSelectionImageRegion3DSerializer(serializers.ModelSerializer):
    """image selection image region 3d serializer."""

    class Meta:
        """meta options."""

        model = ImagingSelectionImageRegion3D
        exclude = ["created_at", "updated_at"]


class ImagingSelectionSubjectReferenceSerializer(BaseReferenceModelSerializer):
    """image selection subject reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """meta options."""

        model = ImagingSelectionSubjectReference
        exclude = ["created_at", "updated_at"]


class ImagingSelectionReferenceSerializer(BaseReferenceModelSerializer):
    """image selection reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta."""

        model = ImagingSelectionReference
        exclude = ["created_at", "updated_at"]


class ImagingSelectionSerializer(WritableNestedModelSerializer):
    """image selection serializer."""

    def get_fields(self):
        """Get fields."""
        from dfhir.bodystructures.serializers import (
            BodyStructureCodeableReferenceSerializer,
        )
        from dfhir.imagingstudies.serializers import ImagingStudyReferenceSerializer

        fields = super().get_fields()
        fields["body_site"] = BodyStructureCodeableReferenceSerializer(
            many=True, required=False
        )
        fields["derived_from"] = ImagingStudyReferenceSerializer(required=False)
        return fields

    identifier = IdentifierSerializer(many=True, required=False)
    category = CodeableConceptSerializer(many=True, required=False)
    code = CodeableConceptSerializer(required=False)
    subject = ImagingSelectionSubjectReferenceSerializer(many=False, required=False)
    performer = ImagingSelectionPerformerSerializer(many=True, required=False)
    based_on = ImagingSelectionBasedOnReferenceSerializer(many=True, required=False)
    focus = ImagingSelectionReferenceSerializer(many=True, required=False)
    endpoint = EndpointReferenceSerializer(many=True, required=False)
    instance = ImagingSelectionInstanceSerializer(many=True, required=False)
    image_region_3d = ImagingSelectionImageRegion3DSerializer(many=True, required=False)

    class Meta:
        """Meta."""

        model = ImagingSelection
        exclude = ["created_at", "updated_at"]


class ImagingSelectionCodeablereferenceSerializer(WritableNestedModelSerializer):
    """image selection codeable reference serializer."""

    concept = CodeableConceptSerializer(required=False)
    reference = ImagingSelectionReferenceSerializer(required=False)

    class Meta:
        """Meta."""

        model = ImagingSelectionCodeableReference
        exclude = ["created_at", "updated_at"]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.contrib.postgres.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='CoverageEligibilityRequest',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('status', models.CharField(choices=[('active', 'Active'), ('cancelled', 'Cancelled'), ('draft', 'Draft'), ('entered-in-error', 'Entered in Error')], max_length=255)),
                ('purpose', django.contrib.postgres.fields.ArrayField(base_field=models.CharField(choices=[('auth-requirements', 'Auth Requirements'), ('benefits', 'Benefits'), ('discovery', 'Discovery'), ('validation', 'Validation')], max_length=255), null=True, size=None)),
                ('service_date', models.DateField(null=True)),
                ('created', models.DateTimeField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='CoverageEligibilityRequestEvent',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('when_date_time', models.DateTimeField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='CoverageEligibilityRequestInsurance',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('focal', models.BooleanField(default=False)),
                ('business_arrangement', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='CoverageEligibilityRequestItem',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('supporting_info_sequence', django.contrib.postgres.fields.ArrayField(base_field=models.IntegerField(), null=True, size=None)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='CoverageEligibilityRequestItemDetailReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='CoverageEligibilityRequestItemDiagnosis',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='CoverageEligibilityRequestProviderReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='CoverageEligibilityRequestSupportingInfo',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('sequence', models.IntegerField()),
                ('applies_to_all', models.BooleanField(default=False)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='CoverageEligibilityRequestSupportingInfoInformationReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.contrib.postgres.fields
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0003_initial'),
        ('devices', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='DeviceRequestParameter',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('value_boolean', models.BooleanField(default=False)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DeviceRequestPerformerCodeableReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DeviceRequestPerformerReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DeviceRequestReasonCodeableReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DeviceRequestReasonReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DeviceRequestReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DeviceRequestRequesterCodeableReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DeviceRequestRequesterReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DeviceRequestSubjectReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DeviceRequest',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('instantiates_uri', django.contrib.postgres.fields.ArrayField(base_field=models.URLField(), null=True, size=None)),
                ('status', models.CharField(choices=[('draft', 'Draft'), ('active', 'Active'), ('on-hold', 'On Hold'), ('revoked', 'Revoked'), ('completed', 'Completed'), ('entered-in-error', 'Entered in Error'), ('unknown', 'Unknown')], default='active', max_length=255)),
                ('intent', models.CharField(choices=[('proposal', 'Proposal'), ('plan', 'Plan'), ('order', 'Order'), ('original-order', 'Original Order'), ('reflex-order', 'Reflex Order'), ('filler-order', 'Filler Order'), ('instance-order', 'Instance Order'), ('option', 'Option')], default='order', max_length=255)),
                ('priority', models.CharField(choices=[('routine', 'Routine'), ('urgent', 'Urgent'), ('stat', 'Stat'), ('asap', 'Asap')], default='routine', max_length=255)),
                ('do_not_perform', models.BooleanField(default=False)),
                ('quantity', models.IntegerField(null=True)),
                ('occurrence_datetime', models.DateTimeField(null=True)),
                ('authored_on', models.DateTimeField(null=True)),
                ('as_needed', models.BooleanField(default=False)),
                ('as_needed_for', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_request_as_needed_for', to='base.codeableconcept')),
                ('based_on', models.ManyToManyField(blank=True, related_name='device_request_based_on', to='base.reference')),
                ('code', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='device_request_code', to='devices.devicedevicedefinitioncodeablereference')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



"""Accounts Urls."""

from django.urls import path

from . import views

app_name = "accounts"


urlpatterns = [
    path("accounts/", views.AccountListView.as_view()),
    path("accounts/<int:pk>/", views.AccountDetailView.as_view()),
]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('allergyintolerances', '0005_initial'),
        ('base', '0005_initial'),
        ('conditions', '0004_initial'),
        ('diagnosticreports', '0002_initial'),
        ('documentreferences', '0003_initial'),
        ('familymemberhistories', '0002_initial'),
        ('observations', '0002_initial'),
        ('practitionerroles', '0001_initial'),
        ('practitioners', '0001_initial'),
        ('relatedpersons', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='familymemberhistoryparticipantactorreference',
            name='practitioner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='family_member_history_participant_actor_reference_practitioner', to='practitioners.practitioner'),
        ),
        migrations.AddField(
            model_name='familymemberhistoryparticipantactorreference',
            name='practitioner_role',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='family_member_history_participant_actor_reference_practitioner_role', to='practitionerroles.practitionerrole'),
        ),
        migrations.AddField(
            model_name='familymemberhistoryparticipantactorreference',
            name='related_person',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='family_member_history_participant_actor_reference_related_person', to='relatedpersons.relatedperson'),
        ),
        migrations.AddField(
            model_name='familymemberhistoryparticipant',
            name='actor',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='family_member_history_participant_actor', to='familymemberhistories.familymemberhistoryparticipantactorreference'),
        ),
        migrations.AddField(
            model_name='familymemberhistoryprocedure',
            name='code',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='family_member_history_procedure_code', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='familymemberhistoryprocedure',
            name='note',
            field=models.ManyToManyField(blank=True, related_name='family_member_history_procedure_note', to='base.annotation'),
        ),
        migrations.AddField(
            model_name='familymemberhistoryprocedure',
            name='outcome',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='family_member_history_procedure_outcome', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='familymemberhistoryprocedure',
            name='performed_age',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='family_member_history_procedure_performed_age', to='base.age'),
        ),
        migrations.AddField(
            model_name='familymemberhistoryprocedure',
            name='performed_period',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='family_member_history_procedure_performed_period', to='base.period'),
        ),
        migrations.AddField(
            model_name='familymemberhistoryprocedure',
            name='performed_range',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='family_member_history_procedure_performed_range', to='base.range'),
        ),
        migrations.AddField(
            model_name='familymemberhistory',
            name='procedure',
            field=models.ManyToManyField(blank=True, related_name='family_member_history_procedure', to='familymemberhistories.familymemberhistoryprocedure'),
        ),
        migrations.AddField(
            model_name='familymemberhistoryreasoncodeablereference',
            name='concept',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='family_member_history_reason_codeable_reference_codeable_concept', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='familymemberhistory',
            name='reason',
            field=models.ManyToManyField(blank=True, related_name='family_member_history_reason', to='familymemberhistories.familymemberhistoryreasoncodeablereference'),
        ),
        migrations.AddField(
            model_name='familymemberhistoryreasonreference',
            name='allergy_intolerance',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='family_member_history_reason_reference_allergy_intolerance', to='allergyintolerances.allergyintolerance'),
        ),
        migrations.AddField(
            model_name='familymemberhistoryreasonreference',
            name='condition',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='family_member_history_reason_reference_condition', to='conditions.condition'),
        ),
        migrations.AddField(
            model_name='familymemberhistoryreasonreference',
            name='diagnostic_report',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='family_member_history_reason_reference_diagnostic_report', to='diagnosticreports.diagnosticreport'),
        ),
        migrations.AddField(
            model_name='familymemberhistoryreasonreference',
            name='document_reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='family_member_history_reason_reference_document_reference', to='documentreferences.documentreference'),
        ),
        migrations.AddField(
            model_name='familymemberhistoryreasonreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='family_member_history_reason_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='familymemberhistoryreasonreference',
            name='observation',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='family_member_history_reason_reference_observation', to='observations.observation'),
        ),
        migrations.AddField(
            model_name='familymemberhistoryreasoncodeablereference',
            name='reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='family_member_history_reason_codeable_reference_reference', to='familymemberhistories.familymemberhistoryreasonreference'),
        ),
    ]



"""charge item urls."""

from django.urls import path

from dfhir.chargeitems import views

app_name = "chargeitems"

urlpatterns = [
    path("chargeitems/", views.ChargeItemListView.as_view()),
    path("chargeitems/<int:pk>/", views.ChargeItemDetailView.as_view()),
]



"""risk assessment models."""

from django.db import models

from dfhir.base.models import (
    BaseReference,
    CodeableConcept,
    Identifier,
    Reference,
    TimeStampedModel,
)
from dfhir.riskassessments.choices import RiskAssessmentStatusChoices


class RiskAssessmentPrediction(TimeStampedModel):
    """risk assessment prediction."""

    outcome = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="risk_assessment_predictions_outcome",
    )
    probability_decimal = models.DecimalField(
        max_digits=10, decimal_places=10, null=True
    )
    probability_range = models.ForeignKey(
        "base.Range",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="risk_assessment_predictions_probability_range",
    )
    qualitative_risk = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="risk_assessment_predictions_qualitative_risk",
    )
    relative_risk = models.DecimalField(max_digits=10, decimal_places=10, null=True)
    when_period = models.ForeignKey(
        "base.Period",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="risk_assessment_predictions_when_period",
    )
    when_range = models.ForeignKey(
        "base.Range",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="risk_assessment_predictions_when_range",
    )
    rationale = models.TextField(null=True)


class RiskAssessmentPerformerReference(BaseReference):
    """risk assessment performer reference."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="risk_assessment_performer_references_identifier",
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="risk_assessment_performer_references_patient",
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="risk_assessment_performer_references_practitioner",
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="risk_assessment_performer_references_practitioner_role",
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="risk_assessment_performer_references_related_person",
    )
    device = models.ForeignKey(
        "devices.Device",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="risk_assessment_performer_references_device",
    )


class RiskAssessmentReasonReference(BaseReference):
    """risk assessment reason reference."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="risk_assessment_reason_references_identifier",
    )
    condition = models.ForeignKey(
        "conditions.Condition",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="risk_assessment_reason_references_condition",
    )
    observation = models.ForeignKey(
        "observations.Observation",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="risk_assessment_reason_references_observation",
    )
    diagnostic_report = models.ForeignKey(
        "diagnosticreports.DiagnosticReport",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="risk_assessment_reason_references_diagnostic_report",
    )
    document_reference = models.ForeignKey(
        "documentreferences.DocumentReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="risk_assessment_reason_references_document_reference",
    )


class RiskAssessmentReasonCodeableConcept(TimeStampedModel):
    """risk assessment reason codeable concept."""

    concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="risk_assessment_reason_codeable_concepts_code",
    )
    reference = models.ForeignKey(
        RiskAssessmentReasonReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="risk_assessment_reason_codeable_concepts_reference",
    )


class RiskAssessment(TimeStampedModel):
    """risk assessment model."""

    identifier = models.ManyToManyField(
        Identifier,
        blank=True,
        related_name="risk_assessments_identifier",
    )
    based_on = models.ForeignKey(
        Reference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="risk_assessments_based_on",
    )
    parent = models.ForeignKey(
        Reference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="risk_assessments_parent",
    )
    status = models.CharField(
        max_length=200, null=True, choices=RiskAssessmentStatusChoices.choices
    )
    method = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="risk_assessments_method",
    )
    code = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="risk_assessments_code",
    )
    subject = models.ForeignKey(
        "patients.PatientGroupReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="risk_assessments_subject",
    )
    encounter = models.ForeignKey(
        "encounters.EncounterReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="risk_assessments_encounter",
    )
    occurrence_date_time = models.DateTimeField(null=True)
    occurrence_period = models.ForeignKey(
        "base.Period",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="risk_assessments_occurrence_period",
    )
    condition = models.ForeignKey(
        "conditions.ConditionReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="risk_assessments_condition",
    )
    performer = models.ForeignKey(
        RiskAssessmentPerformerReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="risk_assessments_performer",
    )
    reason = models.ManyToManyField(
        RiskAssessmentReasonCodeableConcept,
        blank=True,
        related_name="risk_assessments_reason",
    )
    basis = models.ManyToManyField(
        Reference,
        blank=True,
        related_name="risk_assessments_basis",
    )
    prediction = models.ManyToManyField(
        RiskAssessmentPrediction, related_name="risk_assessments_prediction", blank=True
    )
    mitigation = models.CharField(max_length=200, null=True)
    note = models.ManyToManyField(
        "base.Annotation", related_name="risk_assessments_note", blank=True
    )



"""Allergy intolerance serializers."""

from drf_writable_nested.serializers import WritableNestedModelSerializer

from dfhir.base.serializers import (
    AgeSerializer,
    AnnotationSerializer,
    BaseReferenceModelSerializer,
    CodeableConceptSerializer,
    IdentifierSerializer,
    PeriodSerializer,
    RangeSerializer,
)
from dfhir.encounters.serializers import EncounterReferenceSerializer
from dfhir.observations.serializers import ObservationCodeableReferenceSerializer
from dfhir.patients.serializers import PatientReferenceSerializer

# from dfhir.observations.serializers import ObservationCodeableReferenceSerializer
from .models import (
    AllergyIntolerance,
    AllergyIntoleranceAsserterReference,
    AllergyIntoleranceReaction,
    AllergyIntoleranceRecorderReference,
    AllergyIntoleranceReference,
)


class AllergyIntoleranceReferenceSerializer(BaseReferenceModelSerializer):
    """Allergy Intolerance Reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta options."""

        model = AllergyIntoleranceReference
        exclude = ["created_at", "updated_at"]


class AllergyIntoleranceRecorderReferenceSerializer(BaseReferenceModelSerializer):
    """Allergy Intolerance Recorder Reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta options."""

        model = AllergyIntoleranceRecorderReference
        exclude = ["created_at", "updated_at"]


class AllergyIntoleranceAsserterReferenceSerializer(BaseReferenceModelSerializer):
    """Allergy Intolerance Asserter Reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta options."""

        model = AllergyIntoleranceAsserterReference
        exclude = ["created_at", "updated_at"]


class AllergyIntoleranceReactionSerializer(WritableNestedModelSerializer):
    """Allergy Intolerance Reaction serializer."""

    substance = CodeableConceptSerializer(many=False, required=False)
    manifestation = ObservationCodeableReferenceSerializer(many=True, required=False)
    exposure_route = CodeableConceptSerializer(many=False, required=False)
    note = AnnotationSerializer(many=True, required=False)

    class Meta:
        """Meta options."""

        model = AllergyIntoleranceReaction
        exclude = ["created_at", "updated_at"]


class AllergyIntoleranceSerializer(WritableNestedModelSerializer):
    """Allergy Intolerance serializer."""

    identifier = IdentifierSerializer(many=True, required=False)
    clinical_status = CodeableConceptSerializer(many=False, required=False)
    verification_status = CodeableConceptSerializer(many=False, required=False)
    type = CodeableConceptSerializer(many=False, required=False)
    code = CodeableConceptSerializer(many=False, required=False)
    patient = PatientReferenceSerializer(many=False, required=False)
    encounter = EncounterReferenceSerializer(many=False, required=False)
    onset_age = AgeSerializer(many=False, required=False)
    onset_period = PeriodSerializer(many=False, required=False)
    onset_range = RangeSerializer(many=False, required=False)
    recorder = AllergyIntoleranceRecorderReferenceSerializer(many=False, required=False)
    asserter = AllergyIntoleranceAsserterReferenceSerializer(many=False, required=False)
    note = AnnotationSerializer(many=True, required=False)
    reaction = AllergyIntoleranceReactionSerializer(many=True, required=False)

    class Meta:
        """Meta options."""

        model = AllergyIntolerance
        exclude = ["created_at", "updated_at"]



"""Substance serializers."""

from drf_writable_nested.serializers import WritableNestedModelSerializer

from dfhir.base.serializers import (
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    IdentifierSerializer,
    RatioSerializer,
)

from .models import (
    Substance,
    SubstanceCodeableReference,
    SubstanceIngredient,
    SubstanceReference,
)


class SubstanceReferenceSerializer(BaseReferenceModelSerializer):
    """Substance Reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = SubstanceReference
        exclude = ["created_at", "updated_at"]


class SubstanceCodeableReferenceSerializer(WritableNestedModelSerializer):
    """Substance Codeable Reference serializer."""

    reference = SubstanceReferenceSerializer(required=False)
    concept = CodeableConceptSerializer(required=False)

    class Meta:
        """Meta class."""

        model = SubstanceCodeableReference
        exclude = ["created_at", "updated_at"]


class SubstanceIngredientSerializer(WritableNestedModelSerializer):
    """Substance Ingredient serializer."""

    quantity = RatioSerializer(required=False)
    substance_codeable_concept = CodeableConceptSerializer(required=False)
    substance_reference = SubstanceReferenceSerializer(required=False)

    class Meta:
        """Meta class."""

        model = SubstanceIngredient
        exclude = ["created_at", "updated_at"]


class SubstanceSerializer(BaseWritableNestedModelSerializer):
    """Substance serializer."""

    identifier = IdentifierSerializer(many=True, required=False)
    category = CodeableConceptSerializer(many=True, required=False)
    ingredient = SubstanceIngredientSerializer(required=False, many=True)

    class Meta:
        """Meta class."""

        model = Substance
        exclude = ["created_at", "updated_at"]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0005_initial'),
        ('invoices', '0002_initial'),
        ('patients', '0001_initial'),
        ('practitionerroles', '0001_initial'),
        ('practitioners', '0001_initial'),
        ('relatedpersons', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='invoiceparticipantactorreference',
            name='practitioner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='invoice_participant_actor_reference_practitioner', to='practitioners.practitioner'),
        ),
        migrations.AddField(
            model_name='invoiceparticipantactorreference',
            name='practitioner_role',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='invoice_participant_actor_reference_practitioner_role', to='practitionerroles.practitionerrole'),
        ),
        migrations.AddField(
            model_name='invoiceparticipantactorreference',
            name='related_person',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='invoice_participant_actor_reference_related_person', to='relatedpersons.relatedperson'),
        ),
        migrations.AddField(
            model_name='invoiceparticipant',
            name='actor',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='invoice_participant_actor', to='invoices.invoiceparticipantactorreference'),
        ),
        migrations.AddField(
            model_name='invoicerecipientreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='invoice_recipient_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='invoicerecipientreference',
            name='organization',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='invoice_recipient_reference_organization', to='base.organizationreference'),
        ),
        migrations.AddField(
            model_name='invoicerecipientreference',
            name='patient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='invoice_recipient_reference_patient', to='patients.patient'),
        ),
        migrations.AddField(
            model_name='invoicerecipientreference',
            name='related_person',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='invoice_recipient_reference_related_person', to='relatedpersons.relatedperson'),
        ),
        migrations.AddField(
            model_name='invoice',
            name='recipient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='invoice_recipient', to='invoices.invoicerecipientreference'),
        ),
    ]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.contrib.postgres.fields
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0001_initial'),
        ('endpoints', '0001_initial'),
        ('organizations', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Position',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('longitude', models.FloatField()),
                ('latitude', models.FloatField()),
                ('altitude', models.FloatField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Location',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('name', models.CharField(max_length=255)),
                ('status', models.CharField(choices=[('active', 'Active'), ('suspended', 'Suspended'), ('inactive', 'Inactive')], default='active', max_length=255)),
                ('alias', django.contrib.postgres.fields.ArrayField(base_field=models.CharField(max_length=255), null=True, size=None)),
                ('description', models.TextField(null=True)),
                ('mode', models.CharField(choices=[('instance', 'Instance'), ('kind', 'Kind')], max_length=255, null=True)),
                ('address', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, to='base.address')),
                ('characteristic', models.ManyToManyField(blank=True, related_name='location_characteristic', to='base.codeableconcept')),
                ('contact', models.ManyToManyField(blank=True, related_name='location_contact', to='base.extendedcontactdetail')),
                ('endpoint', models.ManyToManyField(blank=True, related_name='location_endpoint', to='endpoints.endpointreference')),
                ('form', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='location_form', to='base.codeableconcept')),
                ('hours_of_operation', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, to='base.availability')),
                ('identifier', models.ManyToManyField(blank=True, related_name='location_identifier', to='base.identifier')),
                ('managing_organization', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='location_organization', to='base.organizationreference')),
                ('operational_status', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='location_operational_status', to='base.coding')),
                ('type', models.ManyToManyField(blank=True, related_name='location_type', to='base.codeableconcept')),
                ('virtual_service_details', models.ManyToManyField(blank=True, related_name='location_virtual_service_details', to='base.virtualservicedetails')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='LocationOrganizationReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='location_organization_reference_identifier', to='base.identifier')),
                ('location', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='location_organization_reference', to='locations.location')),
                ('organization', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='location_organization_reference', to='organizations.organization')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='LocationReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='location_reference_identifier', to='base.identifier')),
                ('location', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='location_reference', to='locations.location')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='LocationCodeableReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('concept', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='location_codeable_reference_concept', to='base.codeableconcept')),
                ('reference', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='location_codeable_reference_reference', to='locations.locationreference')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='location',
            name='part_of',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='location_part_of', to='locations.locationreference'),
        ),
        migrations.AddField(
            model_name='location',
            name='position',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, to='locations.position'),
        ),
    ]



"""CoverageEligibilityResponse serializers."""

from drf_writable_nested.serializers import WritableNestedModelSerializer

from dfhir.base.serializers import (
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    IdentifierSerializer,
    MoneySerializer,
    OrganizationReferenceSerializer,
    PeriodSerializer,
)
from dfhir.coverages.serializers import CoverageReferenceSerializer
from dfhir.patients.serializers import PatientReferenceSerializer
from dfhir.practitioners.serializers import (
    PractitionerPractitionerRoleReferenceSerializer,
)

from .models import (
    CoverageEligibilityResponse,
    CoverageEligibilityResponseError,
    CoverageEligibilityResponseEvent,
    CoverageEligibilityResponseInsurance,
    CoverageEligibilityResponseInsuranceItem,
    CoverageEligibilityResponseInsuranceItemBenefit,
    CoverageEligibilityResponseRequesterReference,
)


class CoverageEligibilityResponseErrorSerializer(WritableNestedModelSerializer):
    """Coverage Eligibility Response Error serializer."""

    code = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = CoverageEligibilityResponseError
        exclude = ["created_at", "updated_at"]


class CoverageEligibilityResponseInsuranceItemBenefitSerializer(
    WritableNestedModelSerializer
):
    """Coverage Eligibility Response Insurance Item Benefit serializer."""

    type = CodeableConceptSerializer(many=False, required=False)
    allowed_money = MoneySerializer(many=False, required=False)
    used_money = MoneySerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = CoverageEligibilityResponseInsuranceItemBenefit
        exclude = ["created_at", "updated_at"]


class CoverageEligibilityResponseInsuranceItemSerializer(WritableNestedModelSerializer):
    """Coverage Eligibility Response Insurance Item serializer."""

    category = CodeableConceptSerializer(many=False, required=False)
    product_or_service = CodeableConceptSerializer(many=False, required=False)
    modifier = CodeableConceptSerializer(many=True, required=False)
    provider = PractitionerPractitionerRoleReferenceSerializer(
        many=False, required=False
    )
    network = CodeableConceptSerializer(many=False, required=False)
    unit = CodeableConceptSerializer(many=False, required=False)
    term = CodeableConceptSerializer(many=False, required=False)
    benefit = CoverageEligibilityResponseInsuranceItemBenefitSerializer(
        many=True, required=False
    )
    authorization_supporting = CodeableConceptSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = CoverageEligibilityResponseInsuranceItem
        exclude = ["created_at", "updated_at"]


class CoverageEligibilityResponseInsuranceSerializer(WritableNestedModelSerializer):
    """Coverage Eligibility Response Insurance serializer."""

    coverage = CoverageReferenceSerializer(many=False, required=False)
    benefit_period = PeriodSerializer(many=False, required=False)
    item = CoverageEligibilityResponseInsuranceItemSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = CoverageEligibilityResponseInsurance
        exclude = ["created_at", "updated_at"]


class CoverageEligibilityResponseEventSerializer(WritableNestedModelSerializer):
    """Coverage Eligibility Response Event serializer."""

    type = CodeableConceptSerializer(many=False, required=False)
    when_period = PeriodSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = CoverageEligibilityResponseEvent
        exclude = ["created_at", "updated_at"]


class CoverageEligibilityResponseRequesterReferenceSerializer(
    BaseReferenceModelSerializer
):
    """Coverage Eligibility Response Requester Reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = CoverageEligibilityResponseRequesterReference
        exclude = ["created_at", "updated_at"]


class CoverageEligibilityResponseSerializer(BaseWritableNestedModelSerializer):
    """Coverage Eligibility Response serializer."""

    identifier = IdentifierSerializer(many=True, required=False)
    patient = PatientReferenceSerializer(many=False, required=False)
    event = CoverageEligibilityResponseEventSerializer(many=True, required=False)
    serviced_period = PeriodSerializer(many=False, required=False)
    requestor = CoverageEligibilityResponseRequesterReferenceSerializer(  # codespell:ignore requestor
        many=False, required=False
    )
    insurer = OrganizationReferenceSerializer(many=False, required=False)
    insurance = CoverageEligibilityResponseInsuranceSerializer(
        many=True, required=False
    )
    form = CodeableConceptSerializer(many=False, required=False)
    error = CoverageEligibilityResponseErrorSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = CoverageEligibilityResponse
        exclude = ["created_at", "updated_at"]



"""relatedpersons views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from .models import RelatedPerson
from .serializers import RelatedPersonSerializer


class RelatedPersonListView(APIView):
    """Related person list view."""

    permission_classes = [AllowAny]

    @extend_schema(responses={200: RelatedPersonSerializer(many=True)})
    def get(self, request):
        """Get related person list."""
        related_persons = RelatedPerson.objects.all()
        serializer = RelatedPersonSerializer(related_persons, many=True)
        return Response(serializer.data)

    @extend_schema(
        request=RelatedPersonSerializer, responses={201: RelatedPersonSerializer}
    )
    def post(self, request):
        """Create a related person."""
        serializer = RelatedPersonSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_201_CREATED)


class RelatedPersonDetailView(APIView):
    """Related person detail view."""

    permission_classes = [AllowAny]

    @extend_schema(responses={200: RelatedPersonSerializer})
    def get(self, request, related_person_id):
        """Get related person detail."""
        try:
            related_person = RelatedPerson.objects.get(id=related_person_id)
        except RelatedPerson.DoesNotExist as err:
            raise Http404 from err

        serializer = RelatedPersonSerializer(related_person)
        return Response(serializer.data)

    @extend_schema(
        request=RelatedPersonSerializer, responses={200: RelatedPersonSerializer}
    )
    def put(self, request, related_person_id):
        """Update related person."""
        try:
            related_person = RelatedPerson.objects.get(id=related_person_id)
        except RelatedPerson.DoesNotExist as err:
            raise Http404 from err

        serializer = RelatedPersonSerializer(related_person, data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data)

    @extend_schema(responses={204: None})
    def delete(self, request, related_person_id):
        """Delete related person."""
        try:
            related_person = RelatedPerson.objects.get(id=related_person_id)
        except RelatedPerson.DoesNotExist as err:
            raise Http404 from err

        related_person.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)
